// commands/init.js
const shell = require('shelljs')
const symbols = require('log-symbols')
const clone = require('../utils/clone.js')
const copy = require('../utils/copy.js')
const package = require('../package.json')
const remote = 'https://github.com/UWEBPM/fast-build.git'//模板git仓库地址
const fs =require('fs')
const chalk = require('chalk') // 用于改变控制台输出的文字颜色
const path = require('path')
const ora = require('ora')
const cp = require('child_process')
const inquirer = require('inquirer')
let branch = 'main'// 分支。github在2020.10.1日后新建仓库的默认主分支为main。


/**
 * 对比本地cli版本号与npm远程仓库最新的版本号
 * @param {string} versionNum - npm远程仓库版本号
 * @return {boolean} - 如本地版本号小于npm远程仓库版本号则返回true，否则返回false
 */
let versionContrast = function (versionNum) {
    let isSatisfy = false,
        localVersion = package.version,
        localVersionAry = localVersion.split('.')
        versionNumAry = versionNum.split('.')
    for (let i = 0; i < localVersionAry.length; i++) {
        if (Number(localVersionAry[i]) < Number(versionNumAry[i])) {
            isSatisfy = true
            break
        } else if (Number(localVersionAry[i]) == Number(versionNumAry[i])) {
            isSatisfy = false
            continue
        }
    }
    return isSatisfy
}

const initAction = async (name, option) => {
    // 0. 检查控制台是否可以运行`git `，
    if (!shell.which('git')) {
        console.log(symbols.error, '对不起，git命令不可用！')
        shell.exit(1)
    }
    // 1. 验证输入name是否合法
    if (fs.existsSync(name)) {
        console.log(symbols.warning,`已存在项目文件夹${name}！`)
        return
    }
    if (name.match(/[^A-Za-z0-9\u4e00-\u9fa5_-]/g)) {
        console.log(symbols.error, '项目名称存在非法字符！')
        return
    }
    // 4. 检测更新与下载模板
    const checkVersion = 'npm view uweb-cli version'
    const updateCLI = 'npm update -g uweb-cli'
    shell.exec(checkVersion, {silent:true}, async function (code, stdout, stderr) {
        if (code === 0 && versionContrast(stdout)) {
            console.log(symbols.warning, chalk.yellow(`当前本地CLI版本为${package.version}, 最新版本为${stdout}`));
            inquirer.prompt([
                {
                    type: 'confirm',
                    message: '检测到有新版本CLI，是否更新？',
                    default: 'Y',
                    name: 'isConfirm'
                }
            ]).then((answers) => {
                if (answers.isConfirm) {
                    const installSpinner = ora('正在更新...').start()
                    shell.exec(updateCLI, function (error, stdout, stderr) {
                        if (error) {
                            console.log(symbols.warning, chalk.yellow('更新失败，请尝试手动更新'))
                            installSpinner.fail() // 安装失败
                            shell.exit(1)
                        } else {
                            installSpinner.succeed(chalk.green('更新成功！详情请查看README文档'))
                        }
                    })
                } else {
                    inquirer.prompt([
                        {
                            type: 'confirm',
                            message: '跳过更新CLI并下载模板？',
                            default: 'Y',
                            name: 'isConfirm'
                        }
                    ]).then((answers) => {
                        if (answers.isConfirm) {
                            // copy(name)
                            clone(`direct:${remote}#${branch}`, name, { clone: true })
                        } else {
                            const installSpinner = ora('正在更新...').start()
                            shell.exec(updateCLI, function (error, stdout, stderr) {
                                if (error) {
                                    console.log(symbols.warning, chalk.yellow('更新失败，请尝试手动更新'))
                                    installSpinner.fail() // 安装失败
                                    shell.exit(1)
                                } else {
                                    installSpinner.succeed(chalk.green('更新成功！详情请查看README文档'))
                                }
                            })
                        }
                    })
                }
            })
        } else if (code !== 0) {
            console.log(stderr);
            console.log(symbols.error, chalk.red('检查CLI更新失败'));
            inquirer.prompt([
                {
                    type: 'confirm',
                    message: '检查CLI更新失败，是否直接下载模板?',
                    default: 'Y',
                    name: 'isConfirm'
                }
            ]).then(function (answers) {
                if (answers.isConfirm) {
                    // copy(name)
                    clone(`direct:${remote}#${branch}`, name, { clone: true })
                } else {
                    shell.exit(1)
                }
            })
        } else {
            copy(name)
            // clone(`direct:${remote}#${branch}`, name, { clone: true })
        }
    })
    //自动安装依赖
    // const execProcess = `cd ${name} && npm install`
    // const installSpinner = ora('正在安装依赖...').start()
    // cp.exec(execProcess, function (error, stdout, stderr) {
    //     if (error) {
    //         console.log(symbols.warning, chalk.yellow('自动安装失败，请手动安装！'))
    //         installSpinner.fail() // 安装失败
    //         shell.exit(1)
    //     } else {
    //         //如果成功则直接提示进入目录 执行即可
    //         installSpinner.succeed(chalk.green('依赖安装成功！'))
    //     }
    //     process.exit()
    // })
    // if (shell.exec('npm install').code !== 0) {
    //     console.log(symbols.warning, chalk.yellow('自动安装失败，请手动安装！'))
    //     installSpinner.fail() // 安装失败
    //     shell.exit(1)
    // }
    // installSpinner.succeed(chalk.green('依赖安装成功！'))
}

module.exports = initAction